"""
References:

 - https://github.com/The-Compiler/dotfiles/blob/master/qutebrowser/config.py
 - https://qutebrowser.org/doc/help/configuring.html#configpy
 - https://qutebrowser.org/FAQ.html
"""

## Load autoconfig.yml as set from browser UI
config.load_autoconfig()

# Using this qutebrowser api isn't officially supported yet
import qutebrowser.api.interceptor

REWRITE_URLS = {
    "www.reddit.com": "old.reddit.com",
}

def rewrite_url(request: qutebrowser.api.interceptor.Request):
    cur_url = request.request_url.host()
    if cur_url in REWRITE_URLS:
        request.request_url.setHost(REWRITE_URLS[cur_url])
        try:
            request.redirect(request.request_url)
        except:
            pass

# Redirect some URLs to more friendly equivalents
qutebrowser.api.interceptor.register(rewrite_url)

## Add duplicate key bindings for scrolling
# Ctrl/ctrl/Control/control
config.bind('<Ctrl-y>', 'scroll up')

# C-e is also used to open editor in insert mode
config.bind('<Ctrl-e>', 'scroll down', mode='normal')

# Go to last active tab
config.bind('gl', 'tab-focus last', mode='normal')

#config.bind(',js', 'tsh')
#https://old.reddit.net/r/archlinux/comments/5m2os3/mpv_is_it_possible_to_change_video_quality_while/
config.bind(',mpv', 'spawn mpv --ytdl-format 18 {url}')
config.bind(',Mpv', 'hint links spawn mpv --ytdl-format 18 {hint-url}')



c.new_instance_open_target = 'window'
#c.downloads.position = 'bottom'

## Shorthand to choose search engine
c.url.searchengines['g'] = 'https://www.google.com/search?q={}'
c.url.searchengines['@w'] = 'https://en.wikipedia.org/w/index.php?search={}'
c.url.searchengines['@s'] = 'https://scholar.google.com/scholar?q={}'
c.url.searchengines['@s2'] = 'https://academic.microsoft.com/search?q={}&f=&orderBy=0&skip=0&take=10'
c.url.searchengines['@py'] = 'https://docs.python.org/3/library/{}.html'

## Set default search engine
c.url.searchengines['DEFAULT'] = 'https://lite.duckduckgo.com/lite/?q={}'

## Set external editor
#c.editor.command = ['gvim', '{}']
c.editor.command = ['emacs', '{}']

# Block invalid resource loads
c.content.tls.certificate_errors = "ask-block-thirdparty"

## Javascript stuff
c.content.javascript.enabled = False  # tsh: shortcut to toggle JS
c.content.javascript.can_open_tabs_automatically = False
c.content.javascript.can_access_clipboard = False
#c.content.javascript.can_close_tabs = False

## Don't allow notifications
c.content.notifications.enabled = False

## Don't start playing videos automatically. Data is expensive.
c.content.autoplay = False

## More stuff
#c.content.geolocation = False
#c.content.mouse_lock = False
#c.content.pdfjs = True
#c.content.prefers_reduced_motion = True
#c.content.cookies.store = False

## Auto-complete
c.completion.open_categories = ["quickmarks", "bookmarks",
                                "history", "filesystem"]
c.completion.show = "auto"

## Per-domain settings
with config.pattern("*://.youtube.com") as youtube:
    youtube.content.cookies.accept = "never"

with config.pattern("*//.amazon.*") as amazon:
    amazon.content.cookies.accept = "never"

# Temporary workaround to prevent some websites from crashing qutebrowser
# https://github.com/qutebrowser/qutebrowser/issues/5634
c.qt.workarounds.remove_service_workers = True

## Statusbar
#c.statusbar.widgets[-1] = f"text:TEST"

#config.source("sample.py")
